;##################################################################
;
;   TesBwan for TI-86 (Initialization)
;
;   Programmed by Patrick Davidson (pad@calc.org)
;        
;   This program is in the public domain.  There is no warranty.
;
;   This file was last updated August 24, 2002.
;
;##################################################################

#define __TI85__

#include "ti-85.h"
#include "keys.i"

FIRST_FREE      =$8be1
DELC_LEN        =$800F
textShadow      =TEXT_MEM
_ldHLind        =LD_HL_MHL
_penCol         =CURSOR_X
_curRow         =CURSOR_ROW
_curCol         =CURSOR_COL     

working_map     =$8801
back_buffer     =$e800

interrupt_entry =$8686
interrupt_byte  =$86
interrupt_table =$8700
interrupt_reg   =$87

#include "tesbwan.i"

        .org    $9800

        .db     "TesBwan 0.3 by Patrick D",0

        ld      a,1
        ld      (ZS_BITS),a
        ld      (iy+13),0
        ld      hl,(PROGRAM_ADDR)
        ld      de,memory_initialize-$9800
        add     hl,de
        ld      de,DELC_LEN
        ld      bc,memory_initialize_end-memory_initialize
        ldir
        jp      DELC_LEN

;############## MEMORY SWAPPING CODE (RUN FROM DELC_LEN)

memory_initialize:
        nop
        ld      hl,(PROGRAM_ADDR)     
        ld      de,$9800
        ld      bc,end_of_code-$9800
        call    memory_exchange

        call    main_program

        ld      de,(PROGRAM_ADDR)
        ld      hl,$9800
        ld      bc,end_of_code-$9800

memory_exchange =$-memory_initialize+DELC_LEN

#include "exchange.asm"

memory_initialize_end:

;############## START OF NORMAL CODE (RUN WITH PROGRAM STARTING AT $9800)
;
; This section of code must rearrange memory to make temporary storage
; available, including storage from $E800 to $F7FF for video buffers.
; The first step is to check if necessary free memory exists; this requires
; computing the end of "used" space (last variable data or end of program,
; whichever is greater);
;
; Then the VAT and stack can be backed up into the memory area immediately
; before $E740, and the program run with the SP adjusted.

main_program:
        ld      de,(FIRST_FREE)         ; DE = first byte unused (by VAT)
        ld      hl,end_of_code          ; HL = first byte unused (by code)
        call    CP_HL_DE                ; test HL - DE
        jr      nc,not_past_first_free  ; skip if positive (HL greater)
        ex      de,h l                  ; HL = first byte unused (by VAT)
not_past_first_free:                    ; HL = forst byte unused

        ld      de,$1400                ; DE = size of storage 
        add     hl,de                   ; HL = end of empty space needed
        bit     7,h
        jr      z,out_of_memory
        ld      de,(VAT_END)
        call    CP_HL_DE
        jr      c,enough_mem

out_of_memory:
        call    _clrLCD
        ld      (CURSOR_ROW),de
        ld      hl,no_mem
        call    _puts
loop:   call    GET_KEY
        or      a
        jr      z,loop
        ret

no_mem: .db     "ERROR: 5K OF FREE RAM"
        .db     "REQUIRED!",0

enough_mem:
        di

        xor     a
        ld      hl,$FBFF                ; one byte before last to be saved
        ld      de,(VAT_END)            ; one byte before first to be saved
        sbc     hl,de                   ; size to save
        ld      b,h
        ld      c,l
        ld      hl,$FBFF
        ld      de,$E7FF
        lddr                            ; save data

        ld      hl,$E800-$FC00
        add     hl,sp
        ld      sp,hl                   ; adjust stack pointer

;############## Run the program

        ld      (exit_program+1),sp
resume_program:
        call    main_code

;############## Exit routines

exit_program:
        ld      sp,0
        di
        im      1
        ld      a,$3c
        out     (0),a

        ld      a,(exit_status)
        or      a
        jr      z,just_exit

        dec     a
        call    z,enter_table

        call    show_table
        call    wait_key

        ld      a,(exit_status)
        cp      2
        jr      z,resume_program

just_exit:
        ld      hl,0
        ld      (_curRow),hl
        ld      (iy+13),6

;############## Restore changed stuff

        di

        ld      hl,$FC00-$E800
        add     hl,sp
        ld      sp,hl                   ; adjust stack pointer

        xor     a
        ld      hl,$FBFF                ; one byte before last to be saved
        ld      de,(VAT_END)            ; one byte before first to be saved
        sbc     hl,de                   ; size to save
        ld      b,h
        ld      c,l
        ld      de,$FBFF
        ld      hl,$E7FF
        lddr                            ; save data

        ei
        ret

;############## Data

#include "vars.i"
#include "scores.i"

;############## ROM calls

_clrLCD:
        ROM_CALL(CLEARLCD)
        ret
_vputs:
        ROM_CALL(D_ZM_STR)
        ret
_puts:
        ROM_CALL(D_ZT_STR)
        ret
_putc:
        ROM_CALL(TX_CHARPUT)
        ret

;############## Other source files

#include "lib.asm"
#include "gfx.asm"
#include "info.asm"
#include "main.asm"
#include "drwspr.asm"
#include "drwblock.asm"
#include "paddle.asm"
#include "ball.asm"
#include "bricks.asm"
#include "levels.asm"
#include "bonuses.asm"
#include "title.asm"
#include "scores.asm"

tiles:
#include "tiles.inc"

end_of_code:
        .end
